@@ -128,7 +128,7 @@ ngx_str_t  ngx_http_charset_default_types[] = {
     ngx_string("text/xml"),
     ngx_string("text/plain"),
     ngx_string("text/vnd.wap.wml"),
-    ngx_string("application/x-javascript"),
+    ngx_string("application/javascript"),
     ngx_string("application/rss+xml"),
     ngx_null_string
 };
